Imports System.IO
Imports System.ServiceModel.Channels
Imports System.Threading
Imports Microsoft.ServiceBus
Imports Microsoft.ServiceBus.Description
Imports Microsoft.ServiceBus.Messaging

Namespace MSDNReceiver
	Friend Class Program
		'Windows Azure Service Bus properties
		Private Shared serviceBusNamespace As String = "msdnservicebus"
		Private Shared serviceBusIssuerName As String = "owner"
		Private Shared serviceBusIssuerKey As String = "B5cNsviiKqrGfQcuUD4jHcVLM6je4aCC31PdrMVBOkI="
		Private Shared topicName As String = "headquarterstopic"

		Private Shared receiverDelay As Integer = 200

		Private Shared colors() As ConsoleColor = { ConsoleColor.Red, ConsoleColor.Green, ConsoleColor.Yellow, ConsoleColor.Cyan, ConsoleColor.Magenta, ConsoleColor.Blue, ConsoleColor.White}

		Shared Sub Main(ByVal args() As String)
			Console.WriteLine("Enter Store Number")
			Dim storeNumber As String = Console.ReadLine

			Console.WriteLine("Selecting Subscription for Store...")

			'Create a Subscription Client to the Topic
			Dim runtimeUri As Uri = ServiceBusEnvironment.CreateServiceUri("sb", serviceBusNamespace, String.Empty)
			Dim tokenProvider As TokenProvider = TokenProvider.CreateSharedSecretTokenProvider(serviceBusIssuerName, serviceBusIssuerKey)
			Dim messagingFactory As MessagingFactory = MessagingFactory.Create(runtimeUri, tokenProvider)
			Dim namespaceManager As New NamespaceManager(runtimeUri, TokenProvider.CreateSharedSecretTokenProvider(serviceBusIssuerName, serviceBusIssuerKey))
			Dim subscriptionClient As SubscriptionClient = messagingFactory.CreateSubscriptionClient(topicName, "Store" & storeNumber.Trim & "Sub", ReceiveMode.PeekLock)

			Console.Clear()
			Console.WriteLine("Ready to receive messages from {0}...", subscriptionClient.Name)

			Dim allSessionsAccepted As Boolean = False
			Do While Not allSessionsAccepted
				Try
					Console.WriteLine("Checking for session...")
					Dim sessionReceiver As MessageSession = subscriptionClient.AcceptMessageSession(TimeSpan.FromSeconds(5))

					ReceiveSessionMessages(sessionReceiver)
					Console.WriteLine("All received on this session")
				Catch e1 As TimeoutException
					Console.WriteLine("Got TimeoutException, no more sessions available")
					allSessionsAccepted = True
				End Try
			Loop

			Console.WriteLine(vbCrLf & "Receiver complete.")
			Console.ReadLine()
		End Sub

		Private Shared Sub ReceiveSessionMessages(ByVal receiver As MessageSession)
			' Read messages from subscription until subscription is empty
			Console.WriteLine("Reading messages from subscription {0}", receiver.Path)
			Console.WriteLine("Receiver Type:" & receiver.GetType.Name)
			Console.WriteLine("Receiver.SessionId = " & receiver.SessionId)

			Dim sessionState As SequenceState = GetState(receiver)
			Dim receivedMessage As BrokeredMessage
			receivedMessage = receiver.Receive
			Do While receivedMessage IsNot Nothing
				Dim sessionId As String = receiver.SessionId
				ProcessMessage(receivedMessage, sessionState, receiver)
				Do While sessionState.GetNextOutOfSequenceMessage <> -1
					'Call back deferred messages
					Console.WriteLine("Calling back for deferred message: Category {0}, Message sequence {1}", receiver.SessionId, sessionState.GetNextSequenceId)
					receivedMessage = receiver.Receive(sessionState.GetNextOutOfSequenceMessage)
					ProcessMessage(receivedMessage, sessionState, receiver)
				Loop

				If receivedMessage.Properties.ContainsKey("IsLastMessageInSession") Then
					Exit Do
				End If

				receivedMessage = receiver.Receive
			Loop

			SetState(receiver, Nothing)
			receiver.Close()
		End Sub

		Private Shared Sub ProcessMessage(ByVal message As BrokeredMessage, ByRef sessionState As SequenceState, Optional ByVal session As MessageSession = Nothing)
			If session IsNot Nothing Then
				Dim messageId As Integer = Convert.ToInt32(message.Properties("CatalogPage"))
				If sessionState.GetNextSequenceId = messageId Then
					OutputMessageInfo("RECV: ", message, "State: " & "RECEIVED")
                    sessionState.SetNextSequenceId(messageId + 1)
					message.Complete()
					SetState(session, sessionState)
				Else
					Console.WriteLine("Deferring message: Category {0}, Catalog Page {1}", session.SessionId, messageId)
					sessionState.AddOutOfSequenceMessage(messageId, message.SequenceNumber)
					message.Defer()
					SetState(session, sessionState)
				End If
			End If

			Thread.Sleep(receiverDelay)
		End Sub

		Private Shared Function GetState(ByVal session As MessageSession) As SequenceState
			Dim state As New SequenceState
			Dim stream As Stream = session.GetState

			If stream IsNot Nothing Then
				Using stream
					Dim reader As New System.Xml.Serialization.XmlSerializer(GetType(SequenceState))
					state = CType(reader.Deserialize(stream), SequenceState)
				End Using
			End If

			'Added in case the session state is corrupted
			If state Is Nothing Then
				state = New SequenceState
			End If

			Return state
		End Function

		Private Shared Sub SetState(ByVal session As MessageSession, ByVal state As SequenceState)
			Using stream = New MemoryStream
				Dim XMLwriter As New System.Xml.Serialization.XmlSerializer(GetType(SequenceState))
				XMLwriter.Serialize(stream, state)
				stream.Position = 0
				session.SetState(stream)
			End Using
		End Sub

		Public Shared Sub OutputMessageInfo(ByVal action As String, ByVal message As BrokeredMessage, Optional ByVal additionalText As String = "")
			If message Is Nothing Then
				Exit Sub
			End If
			Dim sequenceId As Integer = Convert.ToInt32(message.Properties("CatalogPage"))
			Dim prop As Object = message.Properties("Category")
			If prop IsNot Nothing Then
				Console.ForegroundColor = colors(Integer.Parse(prop.ToString) Mod colors.Length)
				Console.WriteLine("{0}{1} - {2}", action, message.MessageId, additionalText)
				Console.ResetColor()
			End If
		End Sub
	End Class
End Namespace
