Imports Microsoft.ServiceBus
Imports Microsoft.ServiceBus.Description
Imports Microsoft.ServiceBus.Messaging
Imports System.Threading
Imports System.IO
Imports System.Xml.Serialization

Namespace MSDNSender
	Friend Class Program
		'Windows Azure Service Bus properties
		Private Shared serviceBusNamespace As String = "msdnservicebus"
		Private Shared serviceBusIssuerName As String = "owner"
		Private Shared serviceBusIssuerKey As String = "B5cNsviiKqrGfQcuUD4jHcVLM6je4aCC31PdrMVBOkI="
		Private Shared topicName As String = "headquarterstopic"
		Private Shared senderDelay As Integer = 500

		Private Shared colors() As ConsoleColor = { ConsoleColor.Red, ConsoleColor.Green, ConsoleColor.Yellow, ConsoleColor.Cyan, ConsoleColor.Magenta, ConsoleColor.Blue, ConsoleColor.White}

		Shared Sub Main(ByVal args() As String)
			Console.Title = "MessageSender"

			'Create a Topic client using the Service Bus credentials
			Dim runtimeUri As Uri = ServiceBusEnvironment.CreateServiceUri("sb", serviceBusNamespace, String.Empty)
			Dim tokenProvider As TokenProvider = TokenProvider.CreateSharedSecretTokenProvider(serviceBusIssuerName, serviceBusIssuerKey)
			Dim messagingFactory As MessagingFactory = MessagingFactory.Create(runtimeUri, tokenProvider)
			Dim topicClient As TopicClient = messagingFactory.CreateTopicClient(topicName)

			Console.WriteLine("Preparing to send messages to {0}...", topicClient.Path)
			Console.WriteLine("Sending messages to topic {0}", topicClient.Path)

			'Deserialize XML file with Products, and store them in an object array
			Dim catalog As Catalog = Nothing
			Dim path As String = "ProductsCatalog.xml"
			Dim serializer As New XmlSerializer(GetType(Catalog))
			Dim reader As New StreamReader(path)
			catalog = CType(serializer.Deserialize(reader), Catalog)
			reader.Close()

			'Loop through the array, and send each product in single messages
			Dim numberOfElementsInArray As Integer = catalog.products.Length
			Dim currentArrayElement As Integer = 0
			For Each product As Product In catalog.products
				Dim isLastProductInArray As Boolean = False
                currentArrayElement = currentArrayElement + 1
				If currentArrayElement = numberOfElementsInArray Then
					isLastProductInArray = True
				End If

				Dim message As BrokeredMessage = CreateMessage(catalog.Name, product, isLastProductInArray)
				Try
					topicClient.Send(message)
				Catch e1 As Exception
					Exit For
				End Try

				OutputMessageInfo("SEND: ", message)
				Thread.Sleep(senderDelay)
			Next product

			Console.WriteLine(vbCrLf & "Press [Enter] to exit.")
			Console.ReadLine()
		End Sub

		Private Shared Function CreateMessage(ByVal catalogName As String, ByVal product As Product, ByVal isLastProductInArray As Boolean) As BrokeredMessage
			'Construct message based on properties extracted from the product object
			Dim message As New BrokeredMessage
			message.Properties.Add("Catalog", catalogName)
			message.Properties.Add("ProductId", product.ProductId)
			message.Properties.Add("ProductName", product.ProductName)
			message.Properties.Add("Category", product.Category)
			message.Properties.Add("CatalogPage", product.CatalogPage)
			message.Properties.Add("Store", product.Store)
			If isLastProductInArray Then
				message.Properties.Add("IsLastMessageInSession", "true")
			End If
			message.SessionId = catalogName
			message.MessageId = "Catalog_" & catalogName & "Store_" & product.Store & "ProductName_" & product.ProductName & "Category_" & product.Category & "_CatalogPage_" & product.CatalogPage & "_" & Date.Now.ToShortTimeString
			Return message
		End Function

		Public Shared Sub OutputMessageInfo(ByVal action As String, ByVal message As BrokeredMessage, Optional ByVal additionalText As String = "")
			If message Is Nothing Then
				Exit Sub
			End If
			Dim sequenceId As Integer = Convert.ToInt32(message.Properties("CatalogPage"))
			Dim prop As Object = message.Properties("Category")
			If prop IsNot Nothing Then
				Console.ForegroundColor = colors(Integer.Parse(prop.ToString) Mod colors.Length)
				Console.WriteLine("{0}{1} - {2}", action, message.MessageId, additionalText)
				Console.ResetColor()
			End If
		End Sub
	End Class
End Namespace
